/******************************************************************************
* orthogonal_order.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * orthogonal_order.h
 *
 *  Created on: Jul 6, 2010
 *      Author: pajor
 */

#ifndef ORTHOGONAL_ORDER_H_
#define ORTHOGONAL_ORDER_H_

#include "../types.h"
#include "../dataStructures/graph.h"

#include <boost/graph/graph_traits.hpp>
#include <boost/foreach.hpp>
#include <boost/graph/iteration_macros.hpp>

namespace schematization {


// Retrieve a vector of indices depicting the sorted order of the nodes according to x and y coordinates.
// returns: see above
// I love O(n^2).
template<typename graphType>
vector<size_t> get_orthogonal_order(const orderType &o, const graphType &graph) {
	vector<size_t> out_indices(0);
	vector<bool> deleted(num_vertices(graph), false);

	boundingBox bb = get_graph_bounding_box(graph);

	// Iterate over the nodes, look for minimum, and delete it
	BOOST_FOREACH(typename graphType::vertex_descriptor n, vertices(graph)) {
		if (graph[n].node_id == DUMMY_NODE_ID) continue;
		double current_min = (o == X_ORDER ? bb.right : bb.bottom) * 2 + 100; //100 is a dummy value that ensures that current_min is large enough should bb.right or bb.bottom be 0.
		size_t current_min_index = -1;
		BOOST_FOREACH(typename graphType::vertex_descriptor m, vertices(graph)) {
			if (deleted[m]) continue;

			if ((o == X_ORDER ? graph[m].coord.x : graph[m].coord.y ) < current_min) {
				current_min = o == X_ORDER ? graph[m].coord.x : graph[m].coord.y;
				current_min_index = m;
			}
		}
		assert(current_min_index != -1);

		// push index
		out_indices.push_back(current_min_index);
		//cout << current_min_index << "\t " << graph[current_min_index].coord.x << ", " <<  graph[current_min_index].coord.y << endl; //DEBUG
		
		deleted[current_min_index] = true;
	}


	return out_indices;
}

}

#endif /* ORTHOGONAL_ORDER_H_ */
