/******************************************************************************
* path_extraction.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * path_extraction.h
 *
 *  Created on: Jul 5, 2010
 *      Author: pajor
 */

#ifndef PATH_EXTRACTION_H_
#define PATH_EXTRACTION_H_

#include "../dataStructures/graph.h"
#include "../dataStructures/path.h"

#include <boost/graph/graph_traits.hpp>


namespace schematization {

// This method inserts a path into a given graph G between two nodes.
// Note, that it is assumed that the path passed to this function is
// assumed to contain the source and target nodes itself. Thus, only
// the *inner* nodes of the path are created in G.
template<typename pathType, typename graphType>
void insert_path(pathType &p, typename graphType::vertex_descriptor source_node, typename graphType::vertex_descriptor target_node, graphType &graph) {
	typename graphType::vertex_descriptor prev_node = source_node;

	// Iterate over the inner nodes of the path.
	for (size_t n = 1; n < p.size() - 1; ++n) {
		typename graphType::vertex_descriptor new_node = add_vertex(graph);
		// Copy properties to the node...
		graph[new_node].coord = p[n].coord;

		// Insert edge
		insert_edge(p, prev_node, new_node, graph);

		prev_node = new_node;
	}

	// Insert the last edge.
	insert_edge(p, prev_node, target_node, graph);

}

// This method inserts an edge given a path and source/target vertices.
// Returns whether the edge has been created or not.
template<typename pathType, typename graphType>
bool insert_edge(pathType &p, typename graphType::vertex_descriptor source_node, typename graphType::vertex_descriptor target_node, graphType &graph) {
	bool edge_created;
	typename graphType::edge_descriptor new_edge;
	tie(new_edge, edge_created) = add_edge(source_node, target_node, graph);

	// Copy properties to edge...
	graph[new_edge].name = p.name;
	graph[new_edge].category = p.category;

	return edge_created;
}

}

#endif /* PATH_EXTRACTION_H_ */
