/******************************************************************************
* scaling.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * scaling.h
 *
 *  Created on: Jul 1, 2010
 *      Author: pajor
 */

#ifndef SCALING_H_
#define SCALING_H_

#include "../dataStructures/graph.h"

#include <boost/graph/graph_traits.hpp>

#include <cairo/cairo.h>


namespace schematization {

// This algorithm scales all vertices of a graph to fit into A4 paper.
template<typename graphType>
void scale_graph_to_paper(graphType &graph) {
	// Get bounding box of graph
	boundingBox bb = get_graph_bounding_box(graph);

	// Setup a temporary transform
	cairo_matrix_t trans;
	cairo_matrix_init_identity(&trans);
	const double scale_factor = std::min( (paper_size_w - 2*path_margin)/bb.get_extent_x(), (paper_size_h - 2*path_margin)/bb.get_extent_y());
	cairo_matrix_scale(&trans, scale_factor, scale_factor);
	cairo_matrix_translate(&trans, -bb.left + path_margin / scale_factor, -bb.top + path_margin / scale_factor); 

	// Transform every vertex
	typedef typename graph_traits<graphType>::vertex_iterator vertex_iterator;
	vertex_iterator current_v, last_v;
	for (tie(current_v, last_v) = vertices(graph); current_v != last_v; ++current_v) {
		cairo_matrix_transform_point(&trans, &graph[*current_v].coord.x, &graph[*current_v].coord.y);
	}
}


// This algorithm moves the vertices into the paper bounds
template<typename graphType>
void move_graph_to_paper(graphType &graph) {
	// Get bounding box of graph
	boundingBox bb = get_graph_bounding_box(graph);

	cairo_matrix_t trans;
	cairo_matrix_init_identity(&trans);
	cairo_matrix_translate(&trans, -bb.left + path_margin, -bb.top + path_margin);

	// Transform every point
	typedef typename graph_traits<graphType>::vertex_iterator vertex_iterator;
	vertex_iterator current_v, last_v;
	for (tie(current_v, last_v) = vertices(graph); current_v != last_v; ++current_v) {
		cairo_matrix_transform_point(&trans, &graph[*current_v].coord.x, &graph[*current_v].coord.y);
	}
}

}


#endif /* SCALING_H_ */
