/******************************************************************************
* color.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * color.h
 *
 *  Created on: May 4, 2010
 *      Author: pajor
 */

#ifndef COLOR_H_
#define COLOR_H_

#include <cairo/cairo.h>

namespace schematization {

class color {
public:
	double r,g,b;
	double a;

	color(const double _r, const double _g, const double _b, const double _a = 1) : r(_r), g(_g), b(_b), a(_a) {}
	color() : r(0.0), g(0.0), b(0.0), a(1.0) {}

	inline bool operator==(const color &c) const {
		return r == c.r && g == c.g && b == c.b && a == c.a;
	}
	inline bool operator!=(const color &c) const {
		return r != c.r || g != c.g || b != c.b || a != c.a;
	}

	void set_source_rgba(cairo_t *cr) const {
		cairo_set_source_rgba(cr, r,g,b,a);
	}

	void set_source_rgba(cairo_t *cr, const double override_a) const {
		cairo_set_source_rgba(cr, r,g,b,override_a);
	}


	// Serialization function
	template<class Archive>
	void serialize(Archive &ar, const unsigned int version) {
		ar & r & g & b & a;
	}
};


// Ostream function
ostream &operator<<(ostream &os, const color &c) {
	const char tab('\t');
	return os << c.r << tab << c.g << tab << c.b << tab << c.a;
}

// Istream function
istream &operator>>(istream &is, color &c) {
	return is >> c.r >> c.g >> c.b >> c.a;
}


// Some colors
const color col_lightred(0.95, 0.45, 0.45);
const color col_red(0.8, 0, 0);
const color col_blue(0, 0, 0.8);
const color col_lightblue(0.58, 0.87, 0.92);
const color col_green(0, 0.8, 0);
const color col_black(0, 0, 0);
const color col_grey(0.4, 0.4, 0.4);
const color col_lightgrey(0.8, 0.8, 0.8);
const color col_orange(1.0, 0.5, 0);
const color col_yellow(1.0, 0.8, 0.2);
const color col_white(1.0, 1.0, 1.0);
const color col_ultralightgrey(0.95, 0.95, 0.95);

}

#endif /* COLOR_H_ */
