/******************************************************************************
* graphIO.h
*
* RouteSketch
*
******************************************************************************
* Copyright (C) 2013 Andreas Gemsa <gemsa@kit.edu>, Thomas Pajor
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License as published by the Free
* Software Foundation, either version 3 of the License, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
* more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*****************************************************************************/
/*
 * graphIO.h
 *
 *  Created on: Jul 5, 2010
 *      Author: pajor
 */

#ifndef GRAPHIO_H_
#define GRAPHIO_H_

#include "../dataStructures/graph.h"
#include "../geometry/latLng.h"
#include "../geometry/point.h"
#include "../geometry/tools.h"
#include "../geometry/mercatorProjection.h"

#include <fstream>
#include <algorithm>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/graphviz.hpp>
#include <boost/graph/iteration_macros.hpp>

#include <set>

namespace schematization {

// Writes a boost graph to a graphviz-file
template<typename graphType>
bool write_graph(graphType &graph, const string &filename, bool do_output = true) {
	if (do_output) cout << "Writing graph to " << filename << "... " << flush;
	ofstream f(filename.c_str());

	// See if we could create the file...
	if (!f.is_open()) {
		if (do_output) cout << "FAILED." << endl;
		return false;
	}

	dynamic_properties dp;
	typedef typename vertex_bundle_type<graphType>::type vertex_type;
	typedef typename edge_bundle_type<graphType>::type edge_type;
	dp.property("node_id", get(&vertex_type::node_id, graph));
	dp.property("coord", get(&vertex_type::coord, graph));
	dp.property("category", get(&edge_type::category, graph));
	dp.property("name", get(&edge_type::name, graph));
	dp.property("edge_id", get(&edge_type::edge_id, graph));
	write_graphviz(f, graph, dp);

	f.close();
	if (do_output) cout << "done." << endl;

	return true;
}


// Read a boost graph from a graphviz-file
template<typename graphType>
bool read_graph(graphType &graph, const string &filename, bool do_output = true) {
	if (do_output) cout << "Reading graph from " << filename << "... " << flush;

	ifstream f(filename.c_str());

	if (!f.is_open()) {
		if (do_output) cout << "FAILED." << endl;
		return false;
	}

	dynamic_properties dp;
	typedef typename vertex_bundle_type<graphType>::type vertex_type;
	typedef typename edge_bundle_type<graphType>::type edge_type;
	dp.property("node_id", get(&vertex_type::node_id, graph));
	dp.property("coord", get(&vertex_type::coord, graph));

	dp.property("category", get(&edge_type::category, graph));
	dp.property("name", get(&edge_type::name, graph));
	dp.property("edge_id", get(&edge_type::edge_id, graph));

	read_graphviz(f, graph, dp);

	f.close();
	if (do_output) cout << "done." << endl;
	return true;
}


}

#endif /* GRAPHIO_H_ */
